local function create_flag(player)
    local field = player:field()
    local team = player:team()
    local facing = player:facing()
    local facing_away = player:facing_away()
    local tile = player:current_tile()
    local desired_tile = nil
    local direction_list = {
        facing_away,
        Direction.join(facing_away, Direction.Down),
        Direction.join(facing_away, Direction.Up),
        Direction.Down,
        Direction.Up,
        Direction.join(facing, Direction.Down),
        Direction.join(facing, Direction.Up),
        facing
    }

    for i = 1, #direction_list, 1 do
        local test_tile = tile:get_tile(direction_list[i], 1)
        if test_tile:is_walkable() and #test_tile:find_entities(function() return true end) == 0 then
            desired_tile = test_tile
            break
        end
    end

    if desired_tile ~= nil then
        local flag = Obstacle.new(team)

        flag.can_move_to_func = function(t)
            return false
        end

        flag.owner_list = {};

        local query = function(e)
            if not e then return false end
            if e:health() <= 0 then return false end
            if e:team() ~= flag:team() then return false end
            return true
        end

        flag.on_battle_start_func = function(self)
            flag.owner_list = flag:field():find_players(query)
        end

        flag.on_update_func = function(self)
            for i = 1, #flag.owner_list, 1 do
                if flag.owner_list[i]:health() <= 0 then
                    flag:default_character_delete()
                end
            end
        end

        flag.on_delete_func = function(self)
            for i = 1, #flag.owner_list, 1 do
                flag.owner_list[i]:delete()
            end
            self:erase()
        end

        flag:set_name("Flag")
        flag:set_health(1000)
        flag:set_element(Element.None)
        flag:set_texture(Resources.load_texture("flag.png"), true)
        flag:set_height(60)
        flag:enable_sharing_tile(false)

        flag.defense = DefenseVirusBody.new() -- lua owns this need to keep it alive
        flag:add_defense_rule(flag.defense)

        flag.defense2 = DefenseRule.new(DefensePriority.Last, DefenseOrder.CollisionOnly)
        flag.defense2.filter_statuses_func = function(statuses)
            statuses.flags = statuses.flags & ~Hit.Freeze
            statuses.flags = statuses.flags & ~Hit.Flash
            statuses.flags = statuses.flags & ~Hit.Flinch
            statuses.flags = statuses.flags & ~Hit.Paralyze
            statuses.flags = statuses.flags & ~Hit.Drag
            statuses.element = Element.None
            return statuses
        end

        flag:add_defense_rule(flag.defense2)

        local anim = flag:animation()
        anim:load("flag.animation")
        anim:set_state("DEFAULT")
        anim:set_playback(Playback.Loop)

        -- Check if any flags exist on this team already
        -- We don't want to spawn another if one exists.
        local query = function(c)
            if c:name() == "Flag" and c:team() == team then return true end
        end

        if #field:find_characters(query) == 0 then
            field:spawn(flag, desired_tile)
        end
    end
end

local function find_opponent(player)
    local opponent

    player:field():find_players(function(c)
        if c:team() == player:team() then
            -- already on the same team
            return false
        end

        opponent = c;

        return false
    end)

    return opponent
end

local function spawn_flags(player)
    local opponent = find_opponent(player)

    if not opponent then
        -- no opponent, no flags
        return
    end

    create_flag(opponent)
    create_flag(player)
end

function augment_init(augment)
    local player = augment:owner()
    local component = player:create_component(Lifetimes.Scene)

    local i = 0

    component.on_update_func = function(self)
        i = i + 1
        if i == 2 then
            spawn_flags(player)
            component:eject()
        end
    end
end
